@@ -20,8 +20,9 @@ SVF <- variogram(Gamba,res=3)
 GUESS <- ctmm.guess(Gamba,PROTO,variogram=SVF,interactive=FALSE)
 
 ## ------------------------------------------------------------------------
-control <- list(method="pNewton") # beta optimizer is much faster here
-control <- list(method="pNewton",mc.cores=2) # CRAN policy limits to 2 processes
+# ctmm beta optimizer is more reliable here
+# control <- list(method="pNewton",cores=-1) # use all but 1 core
+control <- list(method="pNewton",cores=2) # CRAN policy limits to 2 processes
 FITS <- ctmm.select(Gamba,GUESS,verbose=TRUE,control=control)
 
 ## ------------------------------------------------------------------------
@@ -44,3 +45,6 @@ title("2 Harmonics")
 ## ------------------------------------------------------------------------
 summary(FITS,IC="BIC")
 
+## ------------------------------------------------------------------------
+summary(FITS,IC="MSPE")
+
